import logging
import os

user_config_dir = os.path.expanduser("~")
logging.basicConfig(filename=user_config_dir + '/.cache/spotui.log', filemode='w',
                    format='%(name)s - %(levelname)s - %(message)s')



import curses
from spotui.src.util import truncate
from spotui.src.menu import Menu
from spotui.src.component import Component


class TracksMenu(Component):
    def __init__(self, stdscr, api, play_track, status=None):
        self.stdscr = stdscr
        self.api = api
        self.currently_playing = None
        self.play_track = play_track
        self.title = "Top Tracks"
        self.interactive = True
        self.items = []
        self.tracks = []
        self.restart()
        self.refresh_now_playing(status)

    def restart(self):
        scry, scrx = self.stdscr.getmaxyx()
        self.startx = round(scrx / 4)
        self.endx = scrx - 2
        self.starty = 0
        self.endy = scry - 5
        self.update_tracks(self.tracks, self.title)

    def update_tracks(self, tracks, title):
        self.tracks = tracks
        self.items = list(map(self.__map_tracks, tracks)) if tracks else []
        self.title = title
        self.component = Menu(
            self.stdscr,
            self.items,
            self.starty,
            self.startx,
            self.endy,
            self.endx,
            self.component and self.component.active,
            self.component.selected if self.component
            and self.component.selected < len(self.items) else 0,
            self.component.scroll_start
            if self.component and self.component.scroll_start else 0,
        )

    def refresh_now_playing(self, status):
        currently_playing = (status["item"]["uri"]
                             if status and status["is_playing"] else None)
        if currently_playing and currently_playing != self.currently_playing:
            self.currently_playing = currently_playing
            self.restart()

    def receive_input(self, key):
        if ((key == curses.KEY_ENTER or key in [10, 13]) and self.items
                and len(self.items) > 0):
            self.items[self.component.selected]["handler"]()
        else:
            self.component.receive_input(key)

    def __map_tracks(self, track):
        available_space = self.endx - self.startx
        highlight = self.currently_playing and self.currently_playing == track[
            "uri"]
        max_word_length = round((available_space / 2) - 3)
        track_name = self.__pad_track_text(
            truncate(track["name"], max_word_length),
            max_word_length,
        )
        artist_name = self.__pad_track_text(
            truncate(track["artist"], max_word_length),
            max_word_length,
        )

        def handler():
            self.play_track(track)

        return {
            "text": track_name + " " + artist_name,
            "handler": handler,
            "type": track['type'],
            "id": track["id"],
            "highlight": highlight,
        }

    def __pad_track_text(self, text, max_word_length):
        spaces_needed = max_word_length - len(text)
        if spaces_needed > 0:
            for i in range(0, spaces_needed + 1):
                text += " "
        return text



import curses
from contextlib import contextmanager, redirect_stderr, redirect_stdout
from os import devnull
from spotui.src.ui import App


@contextmanager
def suppress_stdout_stderr():
    """A context manager that redirects stdout and stderr to devnull"""
    with open(devnull, 'w') as fnull:
        with redirect_stderr(fnull) as err, redirect_stdout(fnull) as out:
            yield (err, out)


def main():
    with suppress_stdout_stderr():
        curses.wrapper(App)


if __name__ == "__main__":
    main()



import os
import spotipy
import spotipy.util as util
from spotui.src.config import get_config
from spotui.src.Logging import logging


class SpotifyApi:
    client = None

    def __init__(self):
        self.auth()

    def auth(self):
        config = get_config()
        user_config_dir = os.path.expanduser("~")
        cache_location = os.path.join(
            user_config_dir, '.config/spotui/SpoTUI_token.cache')
        self.user_name = config.get("spotify_api", "user_name")
        client_id = config.get("spotify_api", "client_id")
        client_secret = config.get("spotify_api", "client_secret")
        redirect_uri = config.get("spotify_api", "redirect_uri")
        scopes = "user-read-playback-state streaming playlist-read-collaborative user-modify-playback-state playlist-modify-public user-library-modify user-top-read user-read-currently-playing playlist-read-private playlist-modify-private user-read-recently-played user-library-read"
        self.token = util.prompt_for_user_token(
            self.user_name,
            scopes,
            client_id=client_id,
            client_secret=client_secret,
            redirect_uri=redirect_uri,
            cache_path=cache_location
        )

        if self.token:
            self.client = spotipy.Spotify(auth=self.token)
        else:
            logging.warning("Can't get token for", self.user_name)

    def get_playing(self):
        try:
            self.auth()
            if not self.token:
                return False
            status = self.client.current_playback()
            return status
        except Exception as e:
            pass

    def search(self, query):
        try:
            self.auth()
            if not self.token:
                return False
            classes = ['track', 'show', 'playlist']
            items = [self.__search_all(c, query) for c in classes]
            flat_items = [item for sublist in items for item in sublist]
            return list(map(self.__map_tracks, flat_items))
        except Exception as e:
            pass

    def start_playback(self,
                       device_id,
                       track_uri=None,
                       context_uri=None,
                       offset=None):
        try:
            self.auth()
            if not self.token:
                return False
            self.client.start_playback(device_id, context_uri, track_uri,
                                       offset)
        except Exception as e:
            pass

    def pause_playback(self, device_id):
        try:
            self.auth()
            if not self.token:
                return False
            self.client.pause_playback(device_id)
        except Exception as e:
            pass

    def previous_track(self, device_id):
        try:
            self.auth()
            if not self.token:
                return False
            self.client.previous_track(device_id)
        except Exception as e:
            pass

    def next_track(self, device_id):
        try:
            self.auth()
            if not self.token:
                return False
            self.client.next_track(device_id)
        except Exception as e:
            pass

    def seek_track(self, device_id, position):
        try:
            self.auth()
            if not self.token:
                return False
            self.client.seek_track(position, device_id)
        except Exception as e:
            pass

    def get_top_tracks(self):
        try:
            self.auth()
            if not self.token:
                return []
            tracks = self.client.current_user_top_tracks()
            items = tracks["items"]
            while tracks["next"]:
                tracks = self.client.next(tracks)
                items += tracks["items"]
            return list(map(self.__map_tracks, items))
        except Exception as e:
            pass

    def get_recently_played(self):
        try:
            self.auth()
            if not self.token:
                return []
            tracks = self.client.current_user_recently_played()
            items = tracks["items"]
            # set used to identify unique tracks, prevent duplication of results
            track_uris = set()
            while tracks["next"]:
                tracks = self.client.next(tracks)
                items += tracks["items"]
                # remove all non-unique track-items from recently played list
                items = [item for item in items if item["track"]["uri"] not in track_uris \
                and (track_uris.add(item["track"]["uri"]) or True)]
            return list(map(self.__map_tracks, items))
        except Exception as e:
            pass

    def show_episodes(self, id):
        try:
            self.auth()
            if not self.token:
                return []
            tracks = self.client.show_episodes(id)
            items = tracks["items"]
            return list(map(self.__map_tracks, items))
        except Exception as e:
            pass

    def get_liked_tracks(self):
        try:
            self.auth()
            if not self.token:
                return []
            tracks = self.client.current_user_saved_tracks()
            items = tracks["items"]
            while tracks["next"]:
                tracks = self.client.next(tracks)
                items += tracks["items"]
            return list(map(self.__map_tracks, items))
        except Exception as e:
            pass

    def get_playlists(self):
        try:
            self.auth()
            if not self.token:
                return []

            # this grabs playlists spotify is trying to get the user to listen to
            # playlists = self.client.user_playlists('spotify')

            playlists = []
            offset = 0

            while True:
                get_playlists = self.client.current_user_playlists(offset=offset)["items"]
                playlists.extend(get_playlists)
                if len(get_playlists) < 50:
                    break
                offset += 50
                

            return list(map(self.__map_playlists, playlists))
        except Exception as e:
            return []
            pass

    def get_playlist_tracks(self, playlist_id):
        try:
            self.auth()
            if not self.token:
                return []
            results = self.client.user_playlist(self.user_name,
                                                playlist_id,
                                                fields="tracks,next")
            tracks = results["tracks"]
            items = tracks["items"]
            while tracks["next"]:
                tracks = self.client.next(tracks)
                items += tracks["items"]
            return list(map(self.__map_tracks, items))
        except Exception as e:
            pass

    def get_devices(self):
        try:
            self.auth()
            if not self.token:
                return []
            devices = self.client.devices()
            return list(map(self.__map_devices, devices["devices"]))
        except Exception as e:
            pass

    def shuffle(self, state):
        try:
            self.auth()
            if not self.token:
                return []
            devices = self.client.shuffle(state)
        except Exception as e:
            pass

    def repeat(self, state):
        try:
            self.auth()
            if not self.token:
                return []
            devices = self.client.repeat(state)
        except Exception as e:
            pass

    def __search_all(self, className, query):
        plural = className + 's'
        tracks = self.__extract_results(self.client.search(
            query, 50, 0, className), plural)
        return tracks

    def __extract_results(self, results, key):
        tracks = results[key]
        items = tracks['items']
        return items

    def __map_tracks(self, item):
        track = item["track"] if "track" in item else item
        return {
            "name": '(' + track['type'] + ') ' + track["name"] if track['type'] != 'track' else track['name'],
            "artist": track["artists"][0]["name"] if track['type'] == 'track' else track['publisher'] if track['type'] == 'show' else ' ',
            # "artist": 'james',
            "type": track['type'],
            "id": track["id"] if track['id'] else False,
            "uri": track["uri"],
        }

    def __map_playlists(self, playlist):
        return {
            "text": playlist["name"],
            "id": playlist["id"],
            "uri": playlist["uri"]
        }

    def __map_devices(self, device):
        return {"text": device["name"], "id": device["id"]}


client = SpotifyApi()



import curses
from spotui.src.util import truncate
from spotui.src.menu import Menu
from spotui.src.component import Component


class PlaylistMenu(Component):
    def __init__(self, stdscr, api, change_tracklist):
        self.stdscr = stdscr
        self.api = api
        self.change_tracklist = change_tracklist
        self.title = "Playlists"
        self.interactive = True
        self.items = self.api.get_playlists()
        self.restart()

    def restart(self):
        self.items = self.api.get_playlists()
        scry, scrx = self.stdscr.getmaxyx()
        self.startx = 0
        self.endx = round(scrx / 4) - 2
        self.starty = 6
        self.endy = scry - 5
        self.component = Menu(
            self.stdscr,
            list(map(self.__map_playlists, self.items)),
            self.starty,
            self.startx,
            self.endy,
            self.endx,
            self.component and self.component.active,
            self.component.selected
            if self.component and self.component.selected else 0,
            self.component.scroll_start
            if self.component and self.component.scroll_start else 0,
        )

    def __select_playlist(self, playlist_name, playlist_id, playlist_uri):
        self.change_tracklist(self.api.get_playlist_tracks(playlist_id),
                              playlist_name, playlist_uri)

    def __map_playlists(self, item):
        available_space = self.endx - self.startx - 6
        item["text"] = truncate(item["text"], available_space)

        def handler():
            self.__select_playlist(item["text"], item["id"], item["uri"])

        item["handler"] = handler
        return item

    def receive_input(self, key):
        if key == curses.KEY_ENTER or key in [10, 13]:
            self.items[self.component.selected]["handler"]()
        else:
            self.component.receive_input(key)



import time


def truncate(text, max_length):
    return (text[:max_length - 3] + "..") if len(text) >= max_length else text


def ms_to_hms(ms):
    if not ms:
        return "00:00"
    seconds = (ms / 1000) % 60
    seconds = int(seconds)
    seconds = str(seconds) if len(str(seconds)) > 1 else "0" + str(seconds)
    minutes = (ms / (1000 * 60)) % 60
    minutes = int(minutes)
    minutes = str(minutes) if len(str(minutes)) > 1 else "0" + str(minutes)
    hours = (ms / (1000 * 60 * 60)) % 24
    hours = int(hours)
    hms = minutes + ":" + seconds
    if hours > 0:
        hours = str(hours) if len(str(hours)) > 1 else "0" + str(hours)
        hms = hours + ":" + hms

    return hms


def debounce(s):
    """Decorator ensures function that can only be called once every `s` seconds.
    """
    def decorate(f):
        t = None

        def wrapped(*args, **kwargs):
            nonlocal t
            t_ = time.time()
            if t is None or t_ - t >= s:
                result = f(*args, **kwargs)
                t = time.time()
                return result

        return wrapped

    return decorate



import curses


def init_colors():
    # Default text
    curses.init_pair(1, curses.COLOR_WHITE, 0)
    # White text
    curses.init_pair(4, curses.COLOR_WHITE, curses.COLOR_BLACK)
    # Yellow text
    curses.init_pair(5, curses.COLOR_YELLOW, curses.COLOR_BLACK)
    # Magenta text
    curses.init_pair(10, curses.COLOR_MAGENTA, curses.COLOR_BLACK)
    # Green text
    curses.init_pair(11, curses.COLOR_GREEN, curses.COLOR_BLACK)
    # Cyan text
    curses.init_pair(12, curses.COLOR_CYAN, curses.COLOR_BLACK)
    # Selected item
    curses.init_pair(6, curses.COLOR_BLACK, curses.COLOR_WHITE)
    # Highlighted (no bg)
    curses.init_pair(7, curses.COLOR_MAGENTA, curses.COLOR_BLACK)
    # Highlighted (bg)
    curses.init_pair(8, curses.COLOR_WHITE, curses.COLOR_MAGENTA)



import curses
from spotui.src.menu import Menu
from spotui.src.component import Component


class LibraryMenu(Component):
    def __init__(self, stdscr, api, change_tracklist):
        self.stdscr = stdscr
        self.api = api
        self.change_tracklist = change_tracklist
        self.title = "Made For You"
        self.interactive = True
        self.items = [
            {
                "text": "Top Tracks",
                "handler": self.__select_top_tracks
            },
            {
                "text": "Recently Played",
                "handler": self.__select_recent_tracks
            },
            {
                "text": "Liked Songs",
                "handler": self.__select_liked_tracks
            },
        ]
        self.restart()

    def restart(self):
        scry, scrx = self.stdscr.getmaxyx()
        self.startx = 0
        self.endx = round(scrx / 4) - 2
        self.starty = 0
        self.endy = 5
        self.component = Menu(
            self.stdscr,
            self.items,
            self.starty,
            self.startx,
            self.endy,
            self.endx,
            self.component and self.component.active,
            self.component.selected
            if self.component and self.component.selected else 0,
            self.component.scroll_start
            if self.component and self.component.scroll_start else 0,
        )

    def __select_top_tracks(self):
        self.change_tracklist(self.api.get_top_tracks(), "Top Tracks")

    def __select_recent_tracks(self):
        self.change_tracklist(self.api.get_recently_played(),
                              "Recently Played")

    def __select_liked_tracks(self):
        self.change_tracklist(self.api.get_liked_tracks(), "Liked Songs")

    def receive_input(self, key):
        if key == curses.KEY_ENTER or key in [10, 13]:
            self.items[self.component.selected]["handler"]()
        else:
            self.component.receive_input(key)
